@@ -34,8 +34,10 @@
 
 #ifdef WITH_SMCLIENT
 #include "eggsmclient.h"
+#ifdef GDK_WINDOWING_X11
 #include "eggdesktopfile.h"
 #endif
+#endif
 
 #include <dbus/dbus-protocol.h>
 #include <dbus/dbus-glib.h>
@@ -85,6 +87,13 @@ terminal_factory_handle_arguments (TerminalFactory *factory,
 #include "terminal-factory-client.h"
 #include "terminal-factory-server.h"
 
+static GType terminal_factory_get_type (void);
+
+G_DEFINE_TYPE_WITH_CODE (TerminalFactory, terminal_factory, G_TYPE_OBJECT,
+  dbus_g_object_type_install_info (g_define_type_id,
+                                   &dbus_glib_terminal_factory_object_info)
+);
+ 
 static void
 terminal_factory_class_init (TerminalFactoryClass *factory_class)
 {
@@ -95,13 +104,6 @@ terminal_factory_init (TerminalFactory *factory)
 {
 }
 
-static GType terminal_factory_get_type (void);
-
-G_DEFINE_TYPE_WITH_CODE (TerminalFactory, terminal_factory, G_TYPE_OBJECT,
-  dbus_g_object_type_install_info (g_define_type_id,
-                                   &dbus_glib_terminal_factory_object_info)
-);
- 
 /* Settings storage works as follows:
  *   /apps/gnome-terminal/global/
  *   /apps/gnome-terminal/profiles/Foo/
@@ -179,19 +181,18 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 
 static void
 about_url_hook (GtkAboutDialog *about,
-	        const char *link,
+	        const char *uri,
 	        gpointer user_data)
 {
   GError *error = NULL;
 
   if (!gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (about)),
-                      link,
+                      uri,
                       gtk_get_current_event_time (),
                       &error))
     {
-      terminal_util_show_error_dialog (GTK_WINDOW (about), NULL,
-                                       _("Could not open link: %s"),
-                                       error->message);
+      terminal_util_show_error_dialog (GTK_WINDOW (about), NULL, error,
+                                       "%s", _("Could not open link"));
       g_error_free (error);
     }
 }
@@ -235,8 +236,8 @@ get_factory_name_for_display (const char *display_name)
 }
 
 /* Evil hack alert: this is exported from libgconf-2 but not in a public header */
-extern gboolean gconf_ping_daemon (void);
-         
+extern gboolean gconf_spawn_daemon(GError** err);
+
 int
 main (int argc, char **argv)
 {
@@ -251,6 +252,9 @@ main (int argc, char **argv)
   DBusGProxy *proxy;
   guint32 request_name_ret;
   GError *error = NULL;
+  const char *home_dir;
+  char *working_directory;
+  int ret = EXIT_SUCCESS;
 
   setlocale (LC_ALL, "");
 
@@ -272,11 +276,14 @@ main (int argc, char **argv)
 
   startup_id = g_getenv ("DESKTOP_STARTUP_ID");
 
-  options = terminal_options_parse (NULL,
+  working_directory = g_get_current_dir ();
+
+  options = terminal_options_parse (working_directory,
                                     NULL,
                                     startup_id,
                                     NULL,
                                     FALSE,
+                                    FALSE,
                                     &argc, &argv,
                                     &error,
                                     gtk_get_option_group (TRUE),
@@ -284,6 +291,9 @@ main (int argc, char **argv)
                                     egg_sm_client_get_option_group (),
 #endif
                                     NULL);
+
+  g_free (working_directory);
+
   if (!options)
     {
       g_printerr (_("Failed to parse arguments: %s\n"), error->message);
@@ -327,7 +337,7 @@ main (int argc, char **argv)
     {
       g_printerr ("Failed to get the session bus: %s\nFalling back to non-factory mode.\n",
                   error->message);
-      g_error_free (error);
+      g_clear_error (&error);
       goto factory_disabled;
     }
 
@@ -351,22 +361,20 @@ main (int argc, char **argv)
                                           &error))
     {
       g_printerr ("Failed name request: %s\n", error->message);
-      g_error_free (error);
+      g_clear_error (&error);
       goto factory_disabled;
     }
 
   /* Forward to the existing factory and exit */
   if (request_name_ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
     {
-      char *working_directory;
       char **env;
       const char *evalue;
       GPtrArray *env_ptr_array;
-      int i, envc;
+      int envc;
       GArray *working_directory_array, *display_name_array, *startup_id_array;
       GArray *env_array, *argv_array;
       gboolean retval;
-      int ret = EXIT_SUCCESS;
 
       _terminal_debug_print (TERMINAL_DEBUG_FACTORY,
                              "Forwarding arguments to existing instance\n");
@@ -417,7 +425,7 @@ main (int argc, char **argv)
             {
               /* Incompatible factory version, fall back, to new instance */
               g_printerr (_("Incompatible factory version; creating a new instance.\n"));
-              g_error_free (error);
+              g_clear_error (&error);
 
               goto factory_disabled;
             }
@@ -447,10 +455,13 @@ factory_disabled:
   /* If the gconf daemon isn't available (e.g. because there's no dbus
    * session bus running), we'd crash later on. Tell the user about it
    * now, and exit. See bug #561663.
+   * Don't use gconf_ping_daemon() here since the server may just not
+   * be running yet, but able to be started. See comments on bug #564649.
    */
-  if (!gconf_ping_daemon ())
+  if (!gconf_spawn_daemon (&error))
     {
-      g_printerr ("Failed to contact the GConf daemon; exiting.\n");
+      g_printerr ("Failed to summon the GConf demon; exiting.  %s\n", error->message);
+      g_error_free (error);
       exit (1);
     }
 
@@ -459,7 +470,7 @@ factory_disabled:
   gtk_about_dialog_set_url_hook (about_url_hook, NULL, NULL);
   gtk_about_dialog_set_email_hook (about_email_hook, NULL, NULL);
 
-#ifdef WITH_SMCLIENT
+#if defined(WITH_SMCLIENT) && defined(GDK_WINDOWING_X11)
   {
     char *desktop_file;
 
@@ -467,7 +478,7 @@ factory_disabled:
                                      "applications",
                                      PACKAGE ".desktop",
                                      NULL);
-    egg_set_desktop_file (desktop_file);
+    egg_set_desktop_file_without_defaults (desktop_file);
     g_free (desktop_file);
   }
 #endif
@@ -475,37 +486,42 @@ factory_disabled:
   terminal_app_initialize (options->use_factory);
   g_signal_connect (terminal_app_get (), "quit", G_CALLBACK (gtk_main_quit), NULL);
 
-  terminal_app_handle_options (terminal_app_get (), options, TRUE /* allow resume */, NULL);
+  terminal_app_handle_options (terminal_app_get (), options, TRUE /* allow resume */, &error);
   terminal_options_free (options);
 
-  /* Now change directory to / so we don't prevent unmounting, e.g. if the
+  if (error)
+    {
+      g_printerr ("Error handling options: %s\n", error->message);
+      g_clear_error (&error);
+
+      ret = EXIT_FAILURE;
+      goto shutdown;
+    }
+
+  /* Now change directory to $HOME so we don't prevent unmounting, e.g. if the
    * factory is started by nautilus-open-terminal. See bug #565328.
+   * On failure back to /.
    */
-  if (chdir ("/") < 0)
-    g_warning ("Failed to chdir to /: %s", g_strerror (errno));
+  home_dir = g_get_home_dir ();
+  if (home_dir == NULL || chdir (home_dir) < 0)
+    (void) chdir ("/");
 
   gtk_main ();
 
+shutdown:
+
   terminal_app_shutdown ();
 
   if (factory)
     g_object_unref (factory);
 
-  return 0;
+  return ret;
 }
 
 /* Factory stuff */
 
 static gboolean
-handle_new_terminal_event (TerminalOptions *options)
-{
-  terminal_app_handle_options (terminal_app_get (), options, FALSE /* no resume */, NULL);
-
-  return FALSE;
-}
-
-static gboolean
-terminal_factory_handle_arguments (TerminalFactory *factory,
+terminal_factory_handle_arguments (TerminalFactory *terminal_factory,
                                    const GArray *working_directory_array,
                                    const GArray *display_name_array,
                                    const GArray *startup_id_array,
@@ -518,6 +534,7 @@ terminal_factory_handle_arguments (TerminalFactory *factory,
   char **env = NULL, **argv = NULL, **argv_copy = NULL;
   int argc;
   GError *arg_error = NULL;
+  gboolean retval;
 
   working_directory = terminal_util_array_to_string (working_directory_array, &arg_error);
   if (arg_error)
@@ -535,6 +552,12 @@ terminal_factory_handle_arguments (TerminalFactory *factory,
   if (arg_error)
     goto out;
 
+  _terminal_debug_print (TERMINAL_DEBUG_FACTORY,
+                         "Factory invoked with working-dir='%s' display='%s' startup-id='%s'\n",
+                         working_directory ? working_directory : "(null)",
+                         display_name ? display_name : "(null)",
+                         startup_id ? startup_id : "(null)");
+
   /* Copy the arguments since terminal_options_parse potentially modifies the array */
   argv_copy = (char **) g_memdup (argv, (argc + 1) * sizeof (char *));
 
@@ -543,6 +566,7 @@ terminal_factory_handle_arguments (TerminalFactory *factory,
                                     startup_id,
                                     env,
                                     TRUE,
+                                    TRUE,
                                     &argc, &argv_copy,
                                     error,
                                     NULL);
@@ -564,10 +588,8 @@ out:
   if (!options)
     return FALSE;
 
-  g_idle_add_full (G_PRIORITY_HIGH_IDLE,
-                   (GSourceFunc) handle_new_terminal_event,
-                   options,
-                   (GDestroyNotify) terminal_options_free);
+  retval = terminal_app_handle_options (terminal_app_get (), options, FALSE /* no resume */, error);
 
-  return TRUE;
+  terminal_options_free (options);
+  return retval;
 }
